/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;

/** Shows basic genetic algorithm on a simle example with displaying crossover and
  * mutation,
  * <p>
  * This is extension of BasicGa. It adds displaying genetic algorithm operators.
  * @see BasicGA
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class BasicGASlow extends Applet 
             implements ActionListener, ItemListener, Runnable
{

  /** Pause between generations in slow run of algorithm */
  final private int SLOW_STEP_PAUSE=1000;

  /** Pause between generations in run of algorithm */
  private int STEP_PAUSE=SLOW_STEP_PAUSE;

  /** Size of population (number of chromosomes), should be even number */
  final private int POP_SIZE=16;
                                 
  /** Thread for running of genetic algorithm in background */
  Thread algThread=null;
                                   
  /** Population of solutions in shown genetic algorithm */
  Population population;

  /** Component for drawing graph and solutions on it */
  GraphCanvas Graph=new GraphCanvas();
  
  CrossoverCanvas PopGraph = new CrossoverCanvas();
  
  /** Button Start */
  Button Start=new Button(" Start ");

  /** Button Step */
  Button Step=new Button(" Step ");

  /** Button Stop */
  Button Stop=new Button(" Stop ");

  /** Button Reset */
  Button Reset=new Button(" Reset ");
  
  Checkbox Fast=new Checkbox("Fast");
  
  boolean fastRun=false;
  

  /** Initialization of applet and genetic algorithm */
  public void init() 
  {
    setBGColor();
    NumberChromosome sample=new NumberChromosome(32);
    sample.randomize();
    sample.countFitness();
    population=new Population(sample, POP_SIZE);
    population.randomize();
    add(PopGraph);
    add(Graph);
    add(Start); 
    Start.addActionListener(this);
    add(Step);
    Step.addActionListener(this);
    add(Stop);  
    Stop.addActionListener(this);
    add(Reset);
    Reset.addActionListener(this);
    add(Fast);
    Fast.addItemListener(this);
    Graph.initPaint(); 
    PopGraph.setPopulation(population);
    PopGraph.animate(!fastRun);
    population.setCrossoverCanvas(PopGraph);
    Graph.setPopulation(population);
  }
    
  /** Sets the background color, which is specified in Applete parameter BGCOLOR.
    * For using, just add to applet tag in HTML something like
    * "<"PARAM name=BGCOLOR value="00ff40">".
    */
  protected void setBGColor()
  {
    try 
      {
        Color bg = new Color(Integer.parseInt(getParameter("BGCOLOR"), 16));
        setBackground(bg);
      } catch (NumberFormatException e) {}
        catch (NullPointerException e) {}
  }

  /** Handles actions from buttons */
  public void actionPerformed(ActionEvent e)
  {
    String command=e.getActionCommand();
    if(command.equals(Start.getLabel())) actionStart();
    if(command.equals(Stop.getLabel())) actionStop();
    if(command.equals(Reset.getLabel())) actionReset();
    if(command.equals(Step.getLabel())) actionStep();
  }

  /** Handles actions from checkboxes and item selectors */
  public void itemStateChanged(ItemEvent e) 
  {
    fastRun=Fast.getState();
    if(fastRun) STEP_PAUSE=0;
           else STEP_PAUSE=SLOW_STEP_PAUSE;
    PopGraph.animate(!fastRun);
  }

  /** Button Start starts genetic algorithm (as a thread) */
  protected void actionStart()
  {
    if(algThread!=null) return;
    algThread = new Thread(this);
    algThread.start();
  }
  
  /** Button Reset resets genetic algorithm */
  protected void actionReset()
  {
    population.stepNumber=0;
    population.randomize();
    Graph.paint();
    PopGraph.paint();
  }
  
  /** Button Stop stops genetic algorithm (the thread) */
  protected void actionStop()
  {
    if(algThread!=null)
      {
        algThread.stop();
        algThread=null;
      }
    Graph.paint();
    PopGraph.paint();
  }
  
  /** Button Step performs one step (on generation) of genetic algorithm */
  protected void actionStep()
  {
    if(fastRun)
      {
        population.newGeneration(80,4);
        Graph.paint();
        PopGraph.paint();
      }
      else
      {
        population.makeStep();
        Graph.paint();
      }
  }
  
  /** Defines run of genetic algorithm thread */
  public void run()
  {
    Graph.setPopulation(population);
    Graph.paint();
    actionStep();
    try { Thread.sleep(STEP_PAUSE);
        } catch (InterruptedException e) {}
    while(true)
      {
         actionStep();   
         try { Thread.sleep(STEP_PAUSE);
             } catch (InterruptedException e) {}
      }
  }

  /** Finishing of the applet and destroying thread. */
  public void destroy()
  {
    if(algThread!=null)
      {
        algThread.stop();
        algThread=null;
      }
    super.destroy();
  }

  /** Finishing of the applet and destroying thread.
    * @exception Throwable successor can throw an exception
    */
  protected void finalize() throws Throwable
  {
    if(algThread!=null)
      {
        algThread.stop();
        algThread=null;
      }
    super.finalize();
  }
  
  /** Returns information about parameters of applet. */
  public String[][] getParameterInfo() 
  {
    String[][] info = 
     {
      {"BGCOLOR", "color (6 hexa digits)", "Background color, for example \"00ff40\"."},
     };
        return info;
  }

  /** Returns information about applet. */
  public String getAppletInfo() 
  {
    return "Title: Genetic algorithm in 2D function with visualization of population\n"+
           "Author: Marek Obitko, obitko@email.cz \n"+
           "Applet shows GA in 2D function. Visualises population forming.";
  }

  /** For running of applet without browser or appletviewer (it is then possible
    * to run this applet as a class directly).
    * @see GAApplication
    */
  public static void main(String[] args)
  {
    Applet applet = new BasicGASlow();
    Frame frame = new GAApplication("Genetic Algorithm", applet, 730, 550);
  }
}