/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

import java.lang.Math;


/** Interface for expression node in a tree with the formula.
  * @see Expression
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
interface ExpressionNode
{
  
  /** Returns value of this node for specified x and y. If there is any subtree,
    * it is evaluated. */
  public double value(double x, double y); 
}



class NumberExpressionNode implements ExpressionNode
{
  private double number;

  public NumberExpressionNode(double n)
  {
    number=n; 
  }
  
  public double value(double x, double y)
  {
    return number; 
  }
}

class XExpressionNode implements ExpressionNode
{
  public double value(double x, double y) 
  {
    return x; 
  }
}

class YExpressionNode implements ExpressionNode
{
  public double value(double x, double y) 
  {
    return y; 
  }
}

class PlusExpressionNode implements ExpressionNode
{
  private ExpressionNode left,right;
  
  public PlusExpressionNode(ExpressionNode l,ExpressionNode r)
  {
    left=l; 
    right=r; 
  }
  
  public double value(double x, double y)
  {
    return left.value(x,y)+right.value(x,y); 
  }
}

class MinusExpressionNode implements ExpressionNode
{
  private ExpressionNode left,right;

  public MinusExpressionNode(ExpressionNode l,ExpressionNode r)
  {
    left=l; 
    right=r; 
  }
  public double value(double x, double y)
  {
    return left.value(x,y)-right.value(x,y); 
  }
}

class MultiplyExpressionNode implements ExpressionNode
{
  private ExpressionNode left,right;
  
  public MultiplyExpressionNode(ExpressionNode l,ExpressionNode r)
  {
    left=l; 
    right=r; 
  }
  
  public double value(double x, double y)
  {
    return left.value(x,y)*right.value(x,y); 
  }
}

class DivideExpressionNode implements ExpressionNode
{
  private ExpressionNode left,right;

  public DivideExpressionNode(ExpressionNode l,ExpressionNode r)
  { 
    left=l; 
    right=r; 
  }
  
  public double value(double x, double y)
  {
    return left.value(x,y)/right.value(x,y); 
  }
}

class PowerExpressionNode implements ExpressionNode
{
  private ExpressionNode left,right;
  
  public PowerExpressionNode(ExpressionNode l,ExpressionNode r)
  {
    left=l; 
    right=r; 
  }
  
  public double value(double x, double y)
  {
    return Math.pow(left.value(x,y),right.value(x,y)); 
  }
}

class UnaryMinusExpressionNode implements ExpressionNode
{
  private ExpressionNode next;
  
  public UnaryMinusExpressionNode(ExpressionNode n)
  {
    next=n; 
  }
  
  public double value(double x, double y)
  {
    return -next.value(x,y); 
  }
}

class FactorialExpressionNode implements ExpressionNode
{
  private ExpressionNode next;
  
  public FactorialExpressionNode(ExpressionNode n)
  { 
    next=n; 
  }
  
  public double value(double x, double y)
  {
	  double v=next.value(x,y);
	  int iv=(int)v;
	  if(iv!=v||iv<0||iv>1000) throw new ArithmeticException();
	  if(iv==0) return 1;
	  int i;
	  double r=1;
	  for(i=1;i<=iv;i++) r*=i;
	  return r;
  }
}

class SinExpressionNode implements ExpressionNode
{
  private ExpressionNode next;
  
  public SinExpressionNode(ExpressionNode n)
  { 
    next=n; 
  }
  
  public double value(double x, double y)
  {
    return Math.sin(next.value(x,y)); 
  }
}

class AsinExpressionNode implements ExpressionNode
{
  private ExpressionNode next;

  public AsinExpressionNode(ExpressionNode n)
  {
    next=n; 
  }
  
  public double value(double x, double y)
  { 
    return Math.asin(next.value(x,y)); 
  }
}

class CosExpressionNode implements ExpressionNode
{
  private ExpressionNode next;

  public CosExpressionNode(ExpressionNode n)
  { 
    next=n; 
  }
  
  public double value(double x, double y)
  {
    return Math.cos(next.value(x,y)); 
  }
}

class AcosExpressionNode implements ExpressionNode
{
  private ExpressionNode next;
  
  public AcosExpressionNode(ExpressionNode n)
  { 
    next=n; 
  }
  
  public double value(double x, double y)
  {
    return Math.acos(next.value(x,y)); 
  }
}

class TanExpressionNode implements ExpressionNode
{
  private ExpressionNode next;

  public TanExpressionNode(ExpressionNode n)
  { 
    next=n; 
  }
  
  public double value(double x, double y)
  {
    return Math.tan(next.value(x,y)); 
  }
}

class AtanExpressionNode implements ExpressionNode
{
  private ExpressionNode next;

  public AtanExpressionNode(ExpressionNode n)
  { 
    next=n; 
  }
  
  public double value(double x, double y)
  {
    return Math.atan(next.value(x,y)); 
  }
}

class SinhExpressionNode implements ExpressionNode
{
  private ExpressionNode next;
  
  public SinhExpressionNode(ExpressionNode n)
  { 
    next=n; 
  }
  
  public double value(double x, double y)
  { 
    double v=next.value(x,y);
	  return (Math.exp(v)-Math.exp(-v))/2; 
  }
}

class AsinhExpressionNode implements ExpressionNode
{
  private ExpressionNode next;

  public AsinhExpressionNode(ExpressionNode n)
  {
    next=n; 
  }
  
  public double value(double x, double y)
  {
    double v=next.value(x,y);
	  return Math.log(v+Math.sqrt(v*v+1));
  }
}

class CoshExpressionNode implements ExpressionNode
{
  private ExpressionNode next;

  public CoshExpressionNode(ExpressionNode n)
  { 
    next=n; 
  }
  
  public double value(double x, double y)
  {
    double v=next.value(x,y);
	  return (Math.exp(v)+Math.exp(-v))/2; 
  }
}

class AcoshExpressionNode implements ExpressionNode
{
  private ExpressionNode next;

  public AcoshExpressionNode(ExpressionNode n)
  {
    next=n; 
  }
  
  public double value(double x, double y)
  {
    double v=next.value(x,y);
	  return Math.log(v+Math.sqrt(v*v-1)); 
  }
}

class TanhExpressionNode implements ExpressionNode
{
  private ExpressionNode next;
  
  public TanhExpressionNode(ExpressionNode n)
  {
    next=n; 
  }
  
  public double value(double x, double y)
  {
    double v=next.value(x,y);
	  return (Math.exp(v)-Math.exp(-v))/(Math.exp(v)+Math.exp(-v)); 
  }
}

class AtanhExpressionNode implements ExpressionNode
{
  private ExpressionNode next;

  public AtanhExpressionNode(ExpressionNode n)
  {
    next=n; 
  }
  
  public double value(double x, double y)
  {
    double v=next.value(x,y);
	  return Math.log((1+v)/(1-v))/2; 
  }
}

class LogExpressionNode implements ExpressionNode
{
  private ExpressionNode next;
  
  public LogExpressionNode(ExpressionNode n)
  {
    next=n; 
  }
  
  public double value(double x, double y)
  {
    double v=next.value(x,y);
	  return Math.log(v)/Math.log(10); 
  }
}

class LnExpressionNode implements ExpressionNode
{
  private ExpressionNode next;

  public LnExpressionNode(ExpressionNode n)
  {
    next=n; 
  }
  
  public double value(double x, double y)
  {
    double v=next.value(x,y);
	  return Math.log(v); 
  }
}

class SqrExpressionNode implements ExpressionNode
{
  private ExpressionNode next;
 
  public SqrExpressionNode(ExpressionNode n)
  { 
    next=n; 
  }
  
  public double value(double x, double y)
  {
    double v=next.value(x,y);
	  return v*v; 
  }
}

class SqrtExpressionNode implements ExpressionNode
{
  private ExpressionNode next;
  
  public SqrtExpressionNode(ExpressionNode n)
  {
    next=n; 
  }
  
  public double value(double x, double y)
  { 
    double v=next.value(x,y);
	  return Math.sqrt(v); 
  }
}

class AbsExpressionNode implements ExpressionNode
{
  private ExpressionNode next;
  
  public AbsExpressionNode(ExpressionNode n)
  {
    next=n; 
  }
  
  public double value(double x, double y)
  {
    double v=next.value(x,y);
	  if(v<0) return -v; else return v; 
  }
}

class IntExpressionNode implements ExpressionNode
{
  private ExpressionNode next;

  public IntExpressionNode(ExpressionNode n)
  { 
    next=n; 
  }
  
  public double value(double x, double y)
  { 
    double v=next.value(x,y);
    return (int)v; 
  }
}

/** Expression in a tree.
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class Expression
{
  
  /** Tree with a formula. */
  private ExpressionNode node;
 
  /** Construct expression in a tree from a string with expression. */
  public Expression(String e) throws ParseError
  {
    LexicalAnalyzer la=new LexicalAnalyzer(e);
    node=analyzeExpression(la);
  } 
  
  /** Returns value of this expression. */
  public double value(double x, double y)
  {
    return node.value(x,y);
  }
  
  /** Analyzes one literal. (x, y, e, pi, or continues with expression.) */
  private static ExpressionNode analyzeLiteral(LexicalAnalyzer la) throws ParseError
  { 
     double v;
     switch(la.type)
     {
       case LexicalAnalyzer.numberType: /* cislo */
             v=la.value;
             la.next();
             return new NumberExpressionNode(v);
       case LexicalAnalyzer.XType: /* x */
             la.next();
             return new XExpressionNode();
       case LexicalAnalyzer.YType: /* y */
             la.next();
             return new YExpressionNode();
       case LexicalAnalyzer.EType: /* e */
             la.next();
             return new NumberExpressionNode(Math.E);
       case LexicalAnalyzer.PIType: /* pi */
             la.next();
             return new NumberExpressionNode(Math.PI);
       case LexicalAnalyzer.leftType: /* (expr) */
             la.next();
             ExpressionNode n=analyzeSum(la);
             if(la.type!=LexicalAnalyzer.rightType)
               la.throwError();
             la.next();
             return n;
           default: la.throwError(); return null;
          } /* switch */
      } /* analyzeLiteral */
  
  /** Analyzes power and factorial. */
  private static ExpressionNode analyzePower(LexicalAnalyzer la) throws ParseError
  {
    ExpressionNode n=analyzeLiteral(la);
    while(true) switch(la.type)
     {
       case LexicalAnalyzer.powerType: /* ^ */
             la.next();
             n=new PowerExpressionNode(n,analyzeLiteral(la));
             break;
	     case LexicalAnalyzer.factorialType: /* ! */
	           la.next();
	           n=new FactorialExpressionNode(n);
	           break;
	     default: return n;
     } /* while a switch */
  } /* analyzePower */

  /** Analyzes function (-, +, sin, asin, cos, acos, tan, atan, sinh, asinh,
    * cosh, acosh, tanh, atanh, log, ln, sqr, sqrt, abs or number). */
  private static ExpressionNode analyzeFunction(LexicalAnalyzer la) throws ParseError
  {
    switch(la.type)
    {
     case LexicalAnalyzer.minusType: /* unarni minus */
       la.next();
       return new UnaryMinusExpressionNode(analyzeFunction(la));
     case LexicalAnalyzer.plusType: /* unarni plus */
       la.next();
       return analyzeFunction(la);
	   case LexicalAnalyzer.sinType: /* sin */
	     la.next();
	     return new SinExpressionNode(analyzeFunction(la));
	   case LexicalAnalyzer.asinType: /* asin */
	     la.next();
	     return new AsinExpressionNode(analyzeFunction(la));
	   case LexicalAnalyzer.cosType: /* cos */
	     la.next();
	     return new CosExpressionNode(analyzeFunction(la));
	   case LexicalAnalyzer.acosType: /* acos */
	     la.next();
	     return new AcosExpressionNode(analyzeFunction(la));
	   case LexicalAnalyzer.tanType: /* tan */
	     la.next();
	     return new TanExpressionNode(analyzeFunction(la));
	   case LexicalAnalyzer.atanType: /* atan */
	     la.next();
	     return new AtanExpressionNode(analyzeFunction(la));
	   case LexicalAnalyzer.sinhType: /* sinh */
	     la.next();
	     return new SinhExpressionNode(analyzeFunction(la));
	   case LexicalAnalyzer.asinhType: /* asinh */
	     la.next();
	     return new AsinhExpressionNode(analyzeFunction(la));
	   case LexicalAnalyzer.coshType: /* cosh */
	     la.next();
	     return new CoshExpressionNode(analyzeFunction(la));
	   case LexicalAnalyzer.acoshType: /* acosh */
	     la.next();
	     return new AcoshExpressionNode(analyzeFunction(la));
	   case LexicalAnalyzer.tanhType: /* tanh */
	     la.next();
	     return new TanhExpressionNode(analyzeFunction(la));
	   case LexicalAnalyzer.atanhType: /* atanh */
	     la.next();
	     return new AtanhExpressionNode(analyzeFunction(la));
	   case LexicalAnalyzer.logType: /* log */
	     la.next();
	     return new LogExpressionNode(analyzeFunction(la));
	   case LexicalAnalyzer.lnType: /* ln */
	     la.next();
	     return new LnExpressionNode(analyzeFunction(la));
	   case LexicalAnalyzer.sqrType: /* sqr */
	     la.next();
	     return new SqrExpressionNode(analyzeFunction(la));
	   case LexicalAnalyzer.sqrtType: /* sqrt */
	     la.next();
	     return new SqrtExpressionNode(analyzeFunction(la));
	   case LexicalAnalyzer.absType: /* abs */
	     la.next();
	     return new AbsExpressionNode(analyzeFunction(la));
	   case LexicalAnalyzer.intType: /* int */
	     la.next();
	     return new IntExpressionNode(analyzeFunction(la));
	   default: return analyzePower(la);
	  } /* switch */
  } /* analyzeFunction */
   
  /** Analyzes * or /. */
  private static ExpressionNode analyzeProduct(LexicalAnalyzer la) throws ParseError
  {
    ExpressionNode n=analyzeFunction(la);
    while(true) switch(la.type)
      {
       case LexicalAnalyzer.multiplyType: /* * */
             la.next();
             n=new MultiplyExpressionNode(n,analyzeFunction(la));
             break;
       case LexicalAnalyzer.divideType: /* / */
             la.next();
             n=new DivideExpressionNode(n,analyzeFunction(la));
             break;
       default: return n;
      } /* while a switch */
  } /* analyzeProduct */

  /** Analyzes + and -. */
  private static ExpressionNode analyzeSum(LexicalAnalyzer la) throws ParseError
  {
    ExpressionNode n=analyzeProduct(la);
    while(true) switch(la.type)
      {
        case LexicalAnalyzer.plusType: /* + */
             la.next();
             n=new PlusExpressionNode(n,analyzeProduct(la));
             break;
        case LexicalAnalyzer.minusType: /* - */
             la.next();
             n=new MinusExpressionNode(n,analyzeProduct(la));
             break;
        default: return n;
      } /* while a switch */
   } /* analyzeSum */
   
  /** Analyzes whole expression. */
  private static ExpressionNode analyzeExpression(LexicalAnalyzer la) throws ParseError
  {
      ExpressionNode n=analyzeSum(la);
      if(la.type!=LexicalAnalyzer.EOIType) la.throwError();
      return n;
  } /* analyzeExpression */
  
} /* class Expression */