/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

/** Graph in 3D space. Mantains points for drawing and all transformation needed.
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class Graph3D
{

  protected final int SPACE_AROUND=50;
  
  public int x[][];
  public int y[][];

  public double dx[][];
  public double dy[][];
  
  public int steps;
  
  public boolean valid;
  
  // rotation
  public double theta=40.0/180.0*Math.PI;
  public double phi=85.0/180.0*Math.PI;
  
  protected int cSize=300;
  
  protected double xMin=-100;
  protected double yMin=-100;
  protected double xMax=100;
  protected double yMax=100;
  
  protected double xScale;
  protected double yScale;
  
  protected double sh;
  protected double xs;
  protected double ys;
  
  protected double xmin;
  protected double ymin;
  protected double xmax;
  protected double ymax;
  
  protected double graphTop=10;
  protected double graphBottom=-10;
  
  public int bigStep=1;
  public int lines=10;
  
  public boolean lookingForMaximum=false;
  
  public Graph3D()
  {
    this(80);
  }
  
  /** @param aSteps number of points (resolution) */
  public Graph3D(int aSteps)
  {
    lines=(int)(aSteps/4);
    resample(aSteps);
    setScale(-100,100,-100,100);
    setSize(300);
  }
  
  /** Sets constraints for graph. */
  public synchronized void setScale(double aXMin, double aXMax, 
                                    double aYMin, double aYMax)
  {
    xMin=aXMin;
    xMax=aXMax;
    yMin=aYMin;
    yMax=aYMax;
    xScale=(xMax-xMin)/steps;
    yScale=(yMax-yMin)/steps;
  }
    
  /** How many lines for each direction. */
  public synchronized void setLines(int aLines)
  {
    lines=aLines;
    bigStep=(int)(steps/lines);
  }
  
  /** Sets size of component for drawing. */
  public synchronized void setSize(int aNewSize)
  {
    cSize=aNewSize;
    sh=cSize/SPACE_AROUND;
    xs=1; // it will be better calculated in recalculate(), this is only approximation
    ys=1;
  }
  
  /** Changes resolution. */
  public synchronized void resample(int aSteps)
  {
    valid=false;
    steps=aSteps;
    x=new int[steps][steps];
    y=new int[steps][steps];
    dx=new double[steps][steps];
    dy=new double[steps][steps];
    bigStep=(int)(steps/lines);
  }
  
  /** Recalculates whole graph. */
  public synchronized void recalculate()
  {
    valid=false;
    double Y,Z;
    double xx,yy,zz;
    int xi,yi;
    xmin=Double.MAX_VALUE;
    xmax=Double.MIN_VALUE;
    ymin=Double.MAX_VALUE;
    ymax=Double.MIN_VALUE;
    graphBottom=Double.MAX_VALUE;
    graphTop=Double.MIN_VALUE;
    double ax,ay;
    for(xi=0; xi<steps; xi++)
      for(yi=0; yi<steps; yi++)
        {
          // possible to optimize ...
          xx=xi*xScale+xMin;
          yy=yi*yScale+yMin;
          
          zz=Function3D.value(xx,yy);
          
          if(zz<graphBottom) graphBottom=zz;
          if(zz>graphTop) graphTop=zz;
          
          Y = xx*Math.sin(theta) + yy*Math.cos(theta);
          Z =(xx*Math.cos(theta) - yy*Math.sin(theta)) * Math.sin(phi) + 
                     zz*Math.cos(phi);
          ax = dx[xi][yi] = Y;
          ay = dy[xi][yi] = Z;
          if(ax<xmin) xmin=ax;
          if(ay<ymin) ymin=ay;
          if(ax>xmax) xmax=ax;
          if(ay>ymax) ymax=ay;
        }
    xs=(cSize-sh*2)/(xmax-xmin);
    ys=(cSize-sh*2)/(ymax-ymin);
    for(xi=0; xi<steps; xi++)
      for(yi=0; yi<steps; yi++)
        {
          x[xi][yi]=(int)//(dx[xi][yi]);
             ( (dx[xi][yi]-xmin)*xs + sh );
          y[xi][yi]=cSize-(int)//(dy[xi][yi]);
             ( (dy[xi][yi]-ymin)*ys + sh);
        }        
    valid=true;
  }

  /** Returns recalculated coordinates.
    * input is x,y in 3D (z is calculated) , output is x,y in 2D.
    * Should call recalculate before xyCoords for every change of graph.
    */
  public synchronized void xyzCoords(double in[], int out[])
  {
    double xx=in[0];
    double yy=in[1];

    double zz=Function3D.value(xx,yy);
          
    double Y = xx*Math.sin(theta) + yy*Math.cos(theta);
    double Z =(xx*Math.cos(theta) - yy*Math.sin(theta)) * Math.sin(phi) + 
               zz*Math.cos(phi);
    out[0]=(int)( (Y-xmin)*xs + sh );
    out[1]=cSize-(int)( (Z-ymin)*ys + sh);
  }

  /** Returns recalculated coordinates.
    * input is x,y,z in 3D (z is NOT calculated) , output is x,y in 2D.
    * Should call recalculate before xyCoords for every change of graph.
    */
  public synchronized void xyCoords(double in[], int out[])
  {
    double xx=in[0];
    double yy=in[1];
    double zz=in[2];

    double Y = xx*Math.sin(theta) + yy*Math.cos(theta);
    double Z =(xx*Math.cos(theta) - yy*Math.sin(theta)) * Math.sin(phi) + 
               zz*Math.cos(phi);
    out[0]=(int)( (Y-xmin)*xs + sh );
    out[1]=cSize-(int)( (Z-ymin)*ys + sh);
  }
  
  /** Returns recalculated coordinates.
    * input is x,y in 2D (z is BOTTOM of graph) , output is x,y in 2D.
    * Should call recalculate before xyCoords for every change of graph.
    */
  public synchronized void xyBottomCoords(long in[], int out[])
  {
    double ni[]=new double[3];
    ni[0]=(xMax-xMin)*((double)1.0*in[0])/4294967296L+xMin;
    ni[1]=(yMax-yMin)*((double)1.0*in[1])/4294967296L+yMin;
    ni[2]=graphBottom;
    xyCoords(ni, out);
  }

  /** Returns recalculated coordinates.
    * input is x,y in 2D (z is TOP of graph) , output is x,y in 2D.
    * Should call recalculate before xyCoords for every change of graph.
    */
  public synchronized void xyTopCoords(long in[], int out[])
  {
    double ni[]=new double[3];
    ni[0]=(xMax-xMin)*((double)1.0*in[0])/4294967296L+xMin;
    ni[1]=(yMax-yMin)*((double)1.0*in[1])/4294967296L+yMin;
    ni[2]=graphTop;
    xyCoords(ni, out);
  }

  /** Returns recalculated coordinates.
    * input is x,y in 2D (z is calculated Z of graph) , output is x,y in 2D.
    * Should call recalculate before xyCoords for every change of graph.
    */
  public synchronized void xyZCoords(long in[], int out[])
  {
    double ni[]=new double[3];
    ni[0]=(xMax-xMin)*((double)1.0*in[0])/4294967296L+xMin;
    ni[1]=(yMax-yMin)*((double)1.0*in[1])/4294967296L+yMin;
    ni[2]=Function3D.value(ni[0],ni[1]);
    xyCoords(ni, out);
  }
  
  /** Returns value of graph function. */
  public synchronized int countFitness(long x, long y)
  {
    double xx=(xMax-xMin)*((double)1.0*x)/4294967296L+xMin;
    double yy=(yMax-yMin)*((double)1.0*y)/4294967296L+yMin;
    double zz=Function3D.value(xx,yy);
	  //System.out.println("x,y,z="+xx+","+yy+","+zz);
    int result=(int)(10000.0*(zz-graphBottom)/(graphTop-graphBottom));
    if(lookingForMaximum) return result;
    return 10010-result;
  }
  
  public double xValue(long x)
  {
    return (double)((xMax-xMin)*((double)1.0*x)/4294967296L+xMin);
  }

  public double yValue(long y)
  {
    return (double)((yMax-yMin)*((double)1.0*y)/4294967296L+yMin);
  }

  public double zValue(long x, long y)
  {
    double xx=(xMax-xMin)*((double)1.0*x)/4294967296L+xMin;
    double yy=(yMax-yMin)*((double)1.0*y)/4294967296L+yMin;
    double zz=Function3D.value(xx,yy);
    return zz;
  }
  
}