/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

import java.awt.TextField;


/** Population of chromosomes in the genetic algorithm.
  * <p>
  * Defines managing of population and creating new generations. It works with abstract class
  * Chromosome, so all you need to use this class is to derive your new chromosome from class
  * Chromosome.
  * @see Chromosome
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class Population
{
  /** size of the population */
  public int size;
  
  /** sample chromosome - used for creating new population etc. */
  public Chromosome sample;
  
  /** the population itself in an array of chromosomes */
  public Chromosome pop[];
  
  /** Sum of all the fitnesses in population - used for roulette wheel selection.
    * @see selectParent
    */
  public int fitnessSum;

  /** Defines if the elitism will be used in creating new generation
    * @see #newGeneration
    */
  public boolean elitism=true;

  /** Population number */
  public int population=0;
  
  /** Text field, where population number is displayed. */
  TextField popNo=null;
  
  /** First parent for crossover */
  public Chromosome parent1;
  
  /** Second parent for crossover */
  public Chromosome parent2;
  
  /** Children from crossover (possibly with mutation) */
  public Chromosome child[];
  
  /** Children from crossover without mutation */
  public Chromosome oldchild[];
  
  /** New population */
  public Chromosome newpop[];
  
  /** Index of parents in population. */
  public int parentIndex[]=new int[2];
  
  /** index in parent index */
  protected int parentIdx=0;
  
  /** step number in slow simulation */
  public int stepNumber=0;

  /** substep number in slow simulation */
  protected int subStepNumber=0;
  
  /** index of last new chromosome in a new population */
  protected int index=0;
  
  /** Crossover canvas for animating genetic algorithm */
  protected CrossoverCanvas canvas=null;
  
  /** Crossover type.
    * @see Chromosome#crossover(Chromosome, double, int)
    */
  public int crossoverType=Chromosome.BASIC_CROSSOVER;
  
  /** Mutation type.
    * @see Chromosome#mute(double, int)
    */
  public int mutationType=Chromosome.BASIC_MUTATION;
  
  /** Creates new population.
    * @param aCromosome defines chromosome from which the population will consist
    * @param asize defines size of the population
    */
  public Population(Chromosome aChromosome, int asize)
  {
    size=asize;
    pop=new Chromosome[size];
    sample=aChromosome; 
    init();
  }

  /** Initializes the population (randomizes all the chromosomes). */
  public void init()
  {
    population=-1;
    updatePopulation();
    for(int i=0; i<size; i++)
      pop[i]=(Chromosome)(sample.copy());
    randomize();
	  countFitnesses();
	  sort();  
	  newpop = new Chromosome[size+2];  // + 2 to prevent overflow
    oldchild=new Chromosome[2];       // for visualisation
  }
  
  /** Counts the fitnesses of all the chromosomes in the population. */
  public void countFitnesses()
  {
    fitnessSum=0;
    for(int i=0; i<size; i++)
      {
        pop[i].countFitness();
        fitnessSum+=pop[i].fitness;
      }  
    //System.out.println(fitnessSum);  
  }

  /** Prints all the chromosomes in the population. */
  public void printSelf()
  {
    for(int i=0; i<size; i++)
      pop[i].printSelf();
  }

  /** Prints the values of all the chromosomes. It is possible to use only for
    * number chromosomes (and derived)!
    * @see NumberChromosome
    */
  public void printValues()
  {
    System.out.println();  
    for(int i=0; i<size; i++)
      System.out.print(((NumberChromosome)pop[i]).value()+",");
    System.out.println();  
  }

  /** Randomizes all the chromosomes in the population. */
  public void randomize()
  {
    population=-1;
    updatePopulation();
    for(int i=0; i<size; i++)
      pop[i].randomize();
    countFitnesses();
    sort();  
  }
  
  /** Sorts the population according to the fitness value. The best is the first. */
  public void sort()
  {
    for(int i=size-1; i>0; i--)
      for(int j=0; j<i; j++)
        if(pop[j].fitness<pop[j+1].fitness)
          {
            Chromosome tmp=pop[j+1];
            pop[j+1]=pop[j];
            pop[j]=tmp;
          }
  }

  /** Creates new generation.
    * @see #newGeneration(int, int)
    */
  public void newGeneration()
  {
    newGeneration(80,5);
  }

  /** Creates new generation with visualization.
    * @see #slowNewGeneration(int, int)
    */
  public void slowNewGeneration()
  {
    slowNewGeneration(80,10);
  }

  /** Selects a parent for crossover. The chromosomes with better fitness are more
    * probable to be selected. So called roulette wheel selection is used.
    */
  public Chromosome selectParent()
  {
    try
     {
      int select=(int)(Math.random()*fitnessSum);
      int sum=0;
      int index=0;
      do
        {
          sum+=pop[index].fitness;
          ++index;
        } while(sum<select);
      index--;
      parentIndex[parentIdx]=index;
      if((++parentIdx)>1) parentIdx=0;
      return pop[index];
     } catch (ArrayIndexOutOfBoundsException e)
       {
         //System.out.println("Wrong array index in selectParent() ");
       }
    return pop[1];  
  }


  /** Creates new generation with specified probabilities of crossover and mutation
    * of new childrens. If the elitism is used, then the best two chromosomes are copied
    * to a new generation.
    * @see elitism
    */
  public void newGeneration(double crossoverProbability, double mutationProbability)
  {
    int i=0; 
    if(elitism)
      {
        newpop[0]=pop[0]; 
        newpop[1]=pop[1];
        i=2;
      }
    for(; i<size; i+=2)
      {
        parent1=selectParent();
        parent2=selectParent();
        child=parent1.crossover(parent2, crossoverProbability, crossoverType);
        child[0].mute(mutationProbability, mutationType);
        child[1].mute(mutationProbability, mutationType);
        newpop[i]=child[0];
        newpop[i+1]=child[1];   
      }
    for(i=0; i<size; i++) pop[i]=newpop[i];
    countFitnesses();
    sort();
    stepNumber=0;
    updatePopulation();
  }

  /** Sets crossover canvas for animation of genetic algorithm */
  public void setCrossoverCanvas(CrossoverCanvas aCanvas)
  {
    canvas=aCanvas;
  }
  
  /** Makes one step in genetic algorithm. Useful for animating GA.
    * @see CrossoverCanvas
    */
  public void makeStep()
  {
    switch(stepNumber)
      {
        case 0: index=0; // elitism or not ?
                subStepNumber=0; 
                if(elitism) stepNumber=1;
                       else stepNumber=2;
                makeStep();
                break;
        case 1: if(index>1) // elitism
                  {
                    stepNumber=2;
                    makeStep();
                  }
                 else 
                  {
                    newpop[index]=pop[index];
                    canvas.makeElitism(++index);
                  }
                break;
        case 2: if(index>=size)  // creating offspring
                  { 
                    stepNumber=3;
                    makeStep();
                  }
                 else
                  {
 	                  switch(subStepNumber)
 	                    {
 	                      case 0: parent1=selectParent();  // select parents
  		                          parent2=selectParent();
  		                          subStepNumber=1;
  		                          child=parent1.crossover(parent2, 80);
                                oldchild[0]=child[0].copy();
                                oldchild[1]=child[1].copy();
                                canvas.showParents();
  		                          break;
  		                  case 1: canvas.showChildren(); // crossover
  		                          subStepNumber=2;
  		                          break;
  		                  case 2: child[0].mute(4);  // mutation
	  	                          child[1].mute(4);
	  	                          canvas.showMutated();
	  	                          subStepNumber=3;
	  	                          break;
	  	                  case 3: newpop[index]=child[0];  // moving to new population
		                            newpop[index+1]=child[1];   
		                            index+=2;
                                canvas.showNewChildren(index);
                                subStepNumber=0;
                                break;
                      }         
                  }
                break;
        case 3: for(int i=0; i<size; i++) pop[i]=newpop[i]; // removing old population
                canvas.removeOldPopulation();
                stepNumber=4;
                break;
        case 4: countFitnesses();  // fitness evaluation and sorting
                sort();
                canvas.showSort();
                updatePopulation();
                stepNumber=0;
                break;
      }
    //canvas.paint();  
  }
  
  /** Creates new generation.
    * @see #newGeneration(int, int)
    */
  public void slowNewGeneration(double crossoverProbability, double mutationProbability)
  {
    int i=0; 
    if(elitism)
      {
        newpop[0]=pop[0]; 
        newpop[1]=pop[1];
        i=2;
      }
    for(; i<size; i+=2)
	    {
	      parent1=selectParent();
  		  parent2=selectParent();
	  	  child=parent1.crossover(parent2, crossoverProbability);
	  	  child[0].mute(mutationProbability);
	  	  child[1].mute(mutationProbability);
        newpop[i]=child[0];
		    newpop[i+1]=child[1];   
	    }
    for(i=0; i<size; i++) pop[i]=newpop[i];
    countFitnesses();
	  sort();
    updatePopulation();
  }

  /** Increments population number and updates text field with it. 
    * @see #popNo
    * @see #population
    */ 
  public void updatePopulation()
  {
    population++;
    if(popNo==null) return;
    popNo.setText(String.valueOf(population));
  }
  
}