/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

import java.awt.Graphics;
import java.awt.Canvas;
import java.awt.Dimension; 
import java.awt.Color;
import java.awt.Image;

/** Component for drawing progress graph.
  * <p> It can draw more graphs in one in different colors.
  * Using is simple - you are just sending new values via method addValue().
  * <p> It uses double buffering to prevent blinking.
  * @see #addValue
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class ProgressGraph extends Canvas
{

  /** Size of component */
  protected Dimension mySize;
  
  /** Graphics of component */
  protected Graphics gr;
  
  /** Graphics of offline buffer */
  protected Graphics offgr;
  
  /** Image of offline buffer */
  protected Image offscreen;
  
  /** Number of colors (graphs) */
  protected int colors;
  
  /** values in the graph */
  protected int val[][];
  
  /** ends of values for each graph */
  protected int end[];

  /** scale for drawing graph */
  protected double scale=10;
    
  /** Constructor which makes standart size graph */
  public ProgressGraph()
  {
    this(690,140,1);
  }
  
  /** Constructor where you can specify size of component and number of graphs */
  public ProgressGraph(int xsize, int ysize, int colorNumber)
  {
    super();
    mySize=new Dimension(xsize,ysize);
    colors=colorNumber;
    val=new int[2*xsize][colors];
    end=new int [colors];
  }

  /** Erases all the graph (in all colors) */
  public void deleteAll()
  {
    for(int i=0;i<colors;i++) end[i]=0;
  }
  
  /** Adds new value to the graph.
    * @param value new value - no scaling is made, so it should be in a range of size of
    *        the component
    * @color number of color (ie. graph)
    */
  public void addValue(int value, int color)
  {
    if((++end[color]) > (2*mySize.width-1)) end[color]=2*mySize.width-1;    
    val[end[color]][color]=value;
    // System.out.print(color+"-"+value+"   ");
  }
  
  /** Draws graph on graphics.
    * @param g graphics which the graph is drawn on
    */
  protected void drawGraph(Graphics g)
  {
    g.setColor(Color.black);  // box around
    g.drawRect(0, 0, mySize.width - 1, mySize.height - 1);
    if(end[0]==0) scale=10; else scale=((double)(mySize.width))/end[0];
    for(int i=0; i<colors; i++)
      {
        switch(i)
          {
            case 0: g.setColor(Color.red);
                    break;
            case 1: g.setColor(Color.blue);
                    break;
            case 2: g.setColor(Color.green);
                    break;
          }  // if you want to use more colors, add some above
        int lasty=val[0][i];
        g.drawLine(1,lasty,1,lasty);
        for(int j=1; j<end[i]; j++)
          {
            //System.out.print(" xxx "+i+" "+j+" "+lasty);
            g.drawLine((int)(j*scale),lasty,(int)((j+1)*scale),val[j][i]);
            lasty=val[j][i];
          }
      }
  }
  
  
  /** Initializes double buffering.
    * This should be called before using method paint.
    * @see #paint
    */
  public void initPaint()
  {
    gr = this.getGraphics();
    offscreen = this.createImage(mySize.width, mySize.height);
    offgr = offscreen.getGraphics();
  }
  
  /** This method acts like repaint(), but it prevents blinking by buffering
    * an image.
    */
  public void paint()
  {
    if(offgr==null) initPaint();
    offgr.setColor(getBackground());
    offgr.fillRect(0, 0, mySize.width, mySize.height);
    offgr.setColor(getForeground());
    paint(offgr);
    gr.drawImage(offscreen, 0, 0, this);
    //paint(getGraphics());
  }
  
  /** Paints graph(s). */
  public void paint(Graphics g) 
  {
     drawGraph(g);  
  }
  
  
  public Dimension getPreferredSize()
  {
    return mySize;
  }

  public Dimension getMaximumSize()
  {
    return mySize;
  }

  public Dimension getMinimumSize()
  {
    return mySize;
  }

}