/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

/** Defines chromosome for solving TSP.
  * <p>
  * Defines all the operators of genetic algorithm except evaluating of fitness function.
  * You should override method countFitness() in your derived classes.
  * Crossover is defined as standart one point crossover.
  * @see TSPGAAll
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class TSPChromosome extends PermutationChromosome
{
  public TSPInstance tspi;
  
  /** Creates new TSP chromosome.
    * @param alength length of the Permutation
    */
  public TSPChromosome(int alength, TSPInstance aTSPInstance)
  {
    super(alength); 
    tspi=aTSPInstance;
  }
  
  /** Fitness is length of the path specified by the chromosome. */
  public void countFitness()
  {
    int sum = 0;
    if(length>0)
      {
        for(int i=0; i<length-1; i++) 
            sum += tspi.distance(perm[i],perm[i+1]);
        sum += tspi.distance(perm[length-1],perm[0]);
      }
    fitness = Integer.MAX_VALUE-sum;
  }
  
  /** Prints the contents of chromosome. */
  public void printSelf()
  {
    System.out.print("TSP: ");
	  for(int i=0; i<length; i++) System.out.print(perm[i]+",");
	  System.out.println();
  }

  /** Returns exact copy of chromosome. */
  public Chromosome copy()
  {
    TSPChromosome result=new TSPChromosome(length, tspi);
    for(int i=0; i<length; i++) result.perm[i]=perm[i];
    result.fitness=fitness;
    result.tspi=tspi;
    return result;
  }

  /** Returns new chromosomes from crossover. Performs one point crossover.
    * Crossover is made of this (called chromosome) and other (parameter).
    * The probability of crosover specifies probability of performing crossover. If the 
    * crossover is not performed, the exact copies of parents are returned. Otherwise
    * one crossover point is randomly chosen and two new childrens are returned.
    * @return array of two new chromosomes - children
    */
  public Chromosome[] crossover(Chromosome other, double probability)
  {
    TSPChromosome[] result=new TSPChromosome[2];
    
    if(probability<(Math.random()*100))  
       { // no crossove 
         lastCrossoverPoint=-1;
         result[0]=(TSPChromosome)((TSPChromosome)this).copy();
         result[1]=(TSPChromosome)((TSPChromosome)other).copy();
       }  
     else
      { // crossover
        result[0]=new TSPChromosome(length,tspi);
        result[1]=new TSPChromosome(length,tspi);
        int crosspoint=(int)(Math.random()*length);  // crossover point
        lastCrossoverPoint=crosspoint;
        int i=0;
        boolean r1u[]=new boolean[length];
        boolean r2u[]=new boolean[length];
        for(i=0; i<length; i++) r1u[i]=r2u[i]=false; 
        for(i=0; i<crosspoint;i++)
          {
            result[0].perm[i]=perm[i];  // copy to crossover point
            r1u[result[0].perm[i]]=true;
            result[1].perm[i]=((TSPChromosome)other).perm[i];
            r2u[result[1].perm[i]]=true;
          }
        int p=crosspoint;
        for(i=0; i<length;i++)
          if(!r1u[i]) result[0].perm[p++]=i;
        p=crosspoint;
        for(i=0; i<length;i++)
          if(!r2u[i]) result[1].perm[p++]=i;
      }
    return result;
  }

  /** Returns new chromosomes from crossover. Performs two point crossover.
    * Crossover is made of this (called chromosome) and other (parameter).
    * The probability of crosover specifies probability of performing crossover. If the 
    * crossover is not performed, the exact copies of parents are returned. Otherwise
    * one crossover point is randomly chosen and two new childrens are returned.
    * @return array of two new chromosomes - children
    */
  public Chromosome[] crossoverRest(Chromosome aother, double probability)
  {
    TSPChromosome[] result=new TSPChromosome[2];
    TSPChromosome other=(TSPChromosome)aother;
    
    if(probability<(Math.random()*100))  
       { // no crossover
         lastCrossoverPoint=-1;
         result[0]=(TSPChromosome)((TSPChromosome)this).copy();
         result[1]=(TSPChromosome)((TSPChromosome)other).copy();
       }  
     else
      { // crossover
        result[0]=new TSPChromosome(length,tspi);
        result[1]=new TSPChromosome(length,tspi);
        int a=(int)(Math.random()*length);  // crossover point
        int b=(int)(Math.random()*length);  // crossover point
        if(a>b) { int p=a; a=b; b=p; }
        lastCrossoverPoint=a;
        boolean used[] = new boolean[length];
        for(int i=0; i<length; i++) used[i]=false;
        for(int i=a; i<b; i++)
          { result[0].perm[i]=perm[i]; used[perm[i]]=true; }
        int index=-1;
        for(int i=0; i<length; i++)
          if(!used[other.perm[i]])
            {
              used[other.perm[i]] = true;
              index++;
              while(index>=a && index<b) index++;
              result[0].perm[index] = other.perm[i];
            }
        for(int i=0; i<length; i++) used[i]=false;
        for(int i=a; i<b; i++)
          { result[1].perm[i]=other.perm[i]; used[other.perm[i]]=true; }
        index=-1;
        for(int i=0; i<length; i++)
          if(!used[perm[i]])
            {
              used[perm[i]] = true;
              index++;
              while(index>=a && index<b) index++;
              result[1].perm[index] = other.perm[i];
            }
      }
    return result;
  }
  
  public static final int REST_CROSSOVER=1;
  public static final int NO_CROSSOVER=2;
  
  public Chromosome[] crossover(Chromosome other, double probability, int crossoverType)
  {
    if(crossoverType==BASIC_CROSSOVER) return crossover(other, probability);
    if(crossoverType==REST_CROSSOVER) return crossoverRest(other, probability);
    if(crossoverType==NO_CROSSOVER) return crossover(other, -1);
    return null;
  }

  public static final int RAND_IMPROVING_MUTATION=1;
  public static final int CONT_IMPROVING_MUTATION=2;
  public static final int RRAND_IMPROVING_MUTATION=3;
  public static final int RCONT_IMPROVING_MUTATION=4;
  public static final int NO_MUTATION=5;
  
  public void mute(double probability, int mutationType)
  {
    if(mutationType==BASIC_MUTATION) mute(probability);
    if(mutationType==RAND_IMPROVING_MUTATION) muteImprovingRand(probability);
    if(mutationType==CONT_IMPROVING_MUTATION) muteImprovingCont(probability);
    if(mutationType==RRAND_IMPROVING_MUTATION) muteImprovingRRand(probability);
    if(mutationType==RCONT_IMPROVING_MUTATION) muteImprovingRCont(probability);
    if(mutationType==NO_MUTATION) return;
  }

  /** Performs random mutation on the chromosome.
    * @param probability specifies probability of switching order at each position
    */
  public void mute(double probability)
  {
    for(int i=0; i<length; i++)
     if((Math.random()*100)<probability)
      {
        int p1=(int)(Math.random()*length);
        int p2=(int)(Math.random()*length);
        int tmp=perm[p1];
        perm[p1]=perm[p2];
        perm[p2]=tmp;
      }
  }   

  /** Performs random and improving mutation on the chromosome.
    * @param probability specifies probability of switching order at each position
    */
  public void muteImprovingRRand(double probability)
  {
    mute(probability);
    muteImprovingRand(probability);
  }
    
  /** Tries to improve by switching cities p1 and p2. If improved, accepts new order. */
  protected void improve(int p1, int p2)
  {
    countFitness();  
    int oldF=fitness;
    int tmp=perm[p1];
    perm[p1]=perm[p2];
    perm[p2]=tmp;
    countFitness();  
    if(fitness<oldF)
      {
        tmp=perm[p1];
        perm[p1]=perm[p2];
        perm[p2]=tmp;
      }
  }
  
  /** Performs only improving random mutation on the chromosome.
    * @param probability specifies probability of switching order at each position
    */
  public void muteImprovingRand(double probability)
  {
    for(int i=0; i<length*2; i++)
      {
        int p1=(int)(Math.random()*length);
        int p2=(int)(Math.random()*length);
        improve(p1,p2);
      }
  }   

  /** Performs random and improving systematic mutation on the chromosome.
    * @param probability specifies probability of switching order at each position
    */
  public void muteImprovingRCont(double probability)
  {
    mute(probability);
    muteImprovingCont(probability);
  }
  
  /** Performs only improving systematic mutation on the chromosome.
    * @param probability specifies probability of switching order at each position
    */
  public void muteImprovingCont(double probability)
  {
    for(int i=0; i<length; i++)
      for(int j=0; j<length; j++)
      {
        int p1=i;
        int p2=j;
        //if(p2==length) p2=0;
        improve(p1,p2);
      }
  }   

  /** Randomizes the chromosome.
    */
  public void randomize()
  {
    for(int i=0; i<length; i++)
      perm[i]=i;
    for(int i=0; i<length/2; i++)
      {
        int p1=(int)(Math.random()*length);
        int p2=(int)(Math.random()*length);
        int tmp=perm[p1];
        perm[p1]=perm[p2];
        perm[p2]=tmp;
      }
  }

}