/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/


import java.awt.AWTEvent;
import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;

/** Component showing cities, path and enabling adding and deleting cities.
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class TSPPathCanvas extends Component implements MouseListener
{

  /** draw city numbers? */
  public boolean DRAW_NUMBERS=true;

    // pathlength = max int - fitness -- HAK
  protected int pathLength;

  /** instance of TSP - coordinates of cities */
  protected TSPInstance tspi;

  /** path - indexes to tspi 
    * @see #tspi */
  protected int path[];

  /** if next city can be added*/
  public boolean canAdd;

  /** if path is drawn */
  public boolean drawPath;

  /** if last user action changed instance of TSP */
  protected boolean eventChanged=false;
  
  protected Dimension mySize;

  /** Graphics of component */
  protected Graphics gr;
  
  /** Graphics of offline buffer */
  protected Graphics offgr;
  
  /** Image of offline buffer */
  protected Image offscreen;
  
  public ActionListener parentApplet=null;
  
  public TSPPathCanvas()
  {
    this(260,270,null);
  }
  
  public TSPPathCanvas(int width, int height)
  {
    this(width, height, null);
  }
  
  /** Creates a new instance of the component.
    * <p> Creates trivial path.
    */
  public TSPPathCanvas(int width, int height, TSPInstance atspi)
  {
    super();
    tspi=atspi;
    mySize=new Dimension(width, height);
    makeTrivialPath();
    canAdd=true;
    //enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    addMouseListener(this);
    drawPath=false;
  }

  public void enableMouseEvents()
  {
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
  }

  /** Creates trivial path. */
  private void makeTrivialPath()
  {
    path=new int[tspi.max];
    for(int i=0; i<tspi.max; i++) path[i]=i;
  }

  /** Changes no of cities. */
  public void changeSize(int newSize)
  {
    tspi.resize(newSize);
    makeTrivialPath();
  }

  /** deletes all cities */
  public void clearCities()
  {
    tspi.clearCities();
    makeTrivialPath();
    canAdd=true;
    drawPath=false;
  }


    // Setting PathLength -- HAK
    //    public void setPathLength(int i)
    //{
    //    pathLength = i;
    // }

    protected void calcPathLength()
    {
        int sum = 0;

        if (! tspi.dmComputed) tspi.computeDistanceMatrix();
        if (tspi.size>1){
            for(int i=0; i<tspi.size; i++) 
                sum += tspi.distance(path[i],path[i+1]);
            sum += tspi.distance(path[tspi.size-1],path[0]);
        }
        pathLength = sum;
  }

  /** randomizes coordinates of all cities. */
  public void randomize()
  {
    randomize(tspi.max);
  }

  /** randomizes coordinates of count cities (and deletes the rest) */
  public void randomize(int count)
  {
    tspi.clearCities();
    for(int i=0; i<count; i++)
      tspi.addCity((int)(Math.random()*(mySize.width-28))+10,
                     (int)(Math.random()*(mySize.height-20))+10);
    drawPath=false;
  }

  /** returns TSP instance */
  public TSPInstance getInstance()
  {
    return tspi;
  }

  /** sets TSP instance */
  public void settspi(TSPInstance atspi)
  {
    tspi=atspi;
    paint();
  }

  /** Sets a new path. If changed, redraws itself. */
  public synchronized void setPath(int apath[])
  {
    boolean eq=true;
    try {
     for(int i=0; i<tspi.size; i++)
        if(apath[i]!=path[i])
          {
            eq=false;
            break;
          }
    if(eq) return;
    for(int i=0; i<tspi.size; i++)
      path[i]=apath[i];
    } catch (ArrayIndexOutOfBoundsException e)
      { /*System.out.println("Error in setPath()");*/ }
    paint();
  }

  /** Initializes double buffering.
    * This should be called before using method paint.
    * @see #paint
    */
  public void initPaint()
  {
    gr = this.getGraphics();
    offscreen = this.createImage(mySize.width, mySize.height);
    offgr = offscreen.getGraphics();
  }
  
  /** This method acts like repaint(), but it prevents blinking by buffering
    * an image.
    */
  public void paint()
  {
    if(offgr==null) initPaint();
    offgr.setColor(getBackground());
    offgr.fillRect(0, 0, mySize.width, mySize.height);
    offgr.setColor(getForeground());
    paint(offgr);
    gr.drawImage(offscreen, 0, 0, this);
    //paint(getGraphics());
  }

  /** Redraws itself.
    * <p> Draws frame, cities (possibly with numbers) and a path.
    * @see #DRAW_NUMBERS
    * @see #drawPath
    */
  public synchronized void paint(Graphics g)
  {
    g.setColor(getBackground());
    g.fillRect(0, 0, 400, 400);
    g.setColor(Color.black);
    g.drawRect(1,1,mySize.width-5,mySize.height-5);

    // Print the pathLength --- HAK
    calcPathLength();
    g.setColor(Color.darkGray);
    g.drawString(""+pathLength,10,20);

    g.setColor(Color.blue);
    for(int i=0; i<tspi.size; i++)
        g.drawOval(tspi.x[i]-2,tspi.y[i]-2,4,4);
    if(DRAW_NUMBERS)
      {
       g.setColor(Color.blue);
       for(int i=0; i<tspi.size; i++)
          g.drawString(""+(i+1),tspi.x[i]+4,tspi.y[i]+1);
      }
    if(drawPath)
      {
       g.setColor(Color.red);
       int ox=tspi.x[path[0]],
           oy=tspi.y[path[0]];
       for(int i=1; i<tspi.size; i++)
          {
            int nx=tspi.x[path[i]],
                ny=tspi.y[path[i]];
            g.drawLine(ox,oy,nx,ny);
            ox=nx; oy=ny;
          }
       g.drawLine(ox,oy,tspi.x[path[0]],tspi.y[path[0]]);

      }
  }

  public Dimension getPreferredSize()
  {
    return mySize;
  }

  public Dimension getMinimumSize()
  {
    return mySize;
  }

  public Dimension getMaximumSize()
  {
    return mySize;
  }

  public int BORDER=1;
  
  public void mouseClicked (MouseEvent e)  
  {
  }
 
  public void mouseEntered (MouseEvent e)  
  {
  }
 
  public void mouseExited  (MouseEvent e)  
  {
  }
 
  public void mousePressed (MouseEvent e)  
  {
    if(canAdd)
     //if(e.getID()==MouseEvent.MOUSE_PRESSED)
       if(tspi.size!=tspi.max)
         if(e.getX()<mySize.width-20*BORDER && e.getX()>10*BORDER &&
            e.getY()<mySize.height-10*BORDER && e.getY()>10*BORDER)
        {
           drawPath=false;
           eventChanged=true;
           tspi.checkCity(e.getX(),e.getY());
            paint();
           if(parentApplet!=null) 
              parentApplet.actionPerformed(new ActionEvent(this,
                 ActionEvent.ACTION_LAST+1, "path"));
        }
  }
  
  public void mouseReleased(MouseEvent e)
  {
  }

  public synchronized boolean changed()
  {
    if(eventChanged) 
      {
        eventChanged=false;
        return true;
      }
    return false;  
  }
  
  public boolean notEnough()
  {
    return tspi.size<3;
  }

}
